@@ -5,7 +5,7 @@ from sklearn.model_selection import StratifiedKFold
 from scipy.stats import binom_test, ks_2samp
 from typing import Callable, Dict, List, Optional, Tuple, Union
 from alibi_detect.base import BaseDetector, concept_drift_dict
-from alibi_detect.cd.utils import update_reference
+from alibi_detect.cd.utils import get_input_shape, update_reference
 from alibi_detect.utils.frameworks import has_pytorch, has_tensorflow
 from alibi_detect.utils.statstest import fdr
 
@@ -21,7 +21,7 @@ logger = logging.getLogger(__name__)
 class BaseClassifierDrift(BaseDetector):
     def __init__(
             self,
-            x_ref: np.ndarray,
+            x_ref: Union[np.ndarray, list],
             p_val: float = .05,
             preprocess_x_ref: bool = True,
             update_x_ref: Optional[Dict[str, int]] = None,
@@ -88,7 +88,7 @@ class BaseClassifierDrift(BaseDetector):
         self.preprocess_x_ref = preprocess_x_ref
         self.update_x_ref = update_x_ref
         self.preprocess_fn = preprocess_fn
-        self.n = x_ref.shape[0]  # type: ignore
+        self.n = len(x_ref)  # type: ignore
 
         # define whether soft preds and optionally the stratified k-fold split
         self.preds_type = preds_type
@@ -104,7 +104,7 @@ class BaseClassifierDrift(BaseDetector):
         self.meta['data_type'] = data_type
         self.meta['params'] = {'binarize_preds ': binarize_preds, 'preds_type': preds_type}
 
-    def preprocess(self, x: np.ndarray) -> Tuple[np.ndarray, np.ndarray]:
+    def preprocess(self, x: Union[np.ndarray, list]) -> Tuple[Union[np.ndarray, list], Union[np.ndarray, list]]:
         """
         Data preprocessing before computing the drift scores.
         Parameters
@@ -122,8 +122,8 @@ class BaseClassifierDrift(BaseDetector):
         else:
             return self.x_ref, x
 
-    def get_splits(self, x_ref: np.ndarray, x: np.ndarray) \
-            -> Tuple[np.ndarray, np.ndarray, List[Tuple[np.ndarray, np.ndarray]]]:
+    def get_splits(self, x_ref: Union[np.ndarray, list], x: Union[np.ndarray, list]) \
+            -> Tuple[Union[np.ndarray, list], np.ndarray, List[Tuple[np.ndarray, np.ndarray]]]:
         """
         Split reference and test data in train and test folds used by the classifier.
 
@@ -136,21 +136,25 @@ class BaseClassifierDrift(BaseDetector):
 
         Returns
         -------
-        List with tuples of train and test indices for optionally different folds.
+        Combined reference and test instances with labels and a list with tuples of
+        train and test indices for optionally different folds.
         """
         # create dataset and labels
-        y = np.concatenate([np.zeros(x_ref.shape[0]), np.ones(x.shape[0])], axis=0).astype(int)
-        x = np.concatenate([x_ref, x], axis=0)
+        y = np.concatenate([np.zeros(len(x_ref)), np.ones(len(x))], axis=0).astype(int)
+        if isinstance(x_ref, np.ndarray) and isinstance(x, np.ndarray):
+            x = np.concatenate([x_ref, x], axis=0)
+        else:  # add 2 lists
+            x = x_ref + x
 
         # random shuffle if stratified folds are not used
+        n_tot = len(x)
         if self.skf is None:
-            n_tot = x.shape[0]
-            idx_shuffle = np.random.choice(np.arange(x.shape[0]), size=n_tot, replace=False)
+            idx_shuffle = np.random.choice(np.arange(n_tot), size=n_tot, replace=False)
             n_tr = int(n_tot * self.train_size)
             idx_tr, idx_te = idx_shuffle[:n_tr], idx_shuffle[n_tr:]
             splits = [(idx_tr, idx_te)]
         else:  # use stratified folds
-            splits = self.skf.split(x, y)
+            splits = self.skf.split(np.zeros(n_tot), y)
         return x, y, splits
 
     def test_probs(
@@ -195,11 +199,12 @@ class BaseClassifierDrift(BaseDetector):
         return p_val, dist
 
     @abstractmethod
-    def score(self, x: np.ndarray) -> Tuple[float, float]:
+    def score(self, x: Union[np.ndarray, list]) -> Tuple[float, float, np.ndarray, np.ndarray]:
         pass
 
-    def predict(self, x: np.ndarray,  return_p_val: bool = True,
-                return_distance: bool = True) -> Dict[Dict[str, str], Dict[str, Union[int, float]]]:
+    def predict(self, x: Union[np.ndarray, list],  return_p_val: bool = True,
+                return_distance: bool = True, return_probs: bool = True) \
+            -> Dict[Dict[str, str], Dict[str, Union[int, float]]]:
         """
         Predict whether a batch of data has drifted from the reference data.
 
@@ -212,16 +217,20 @@ class BaseClassifierDrift(BaseDetector):
         return_distance
             Whether to return a notion of strength of the drift.
             K-S test stat if binarize_preds=False, otherwise relative error reduction.
+        return_probs
+            Whether to return the instance level classifier probabilities for the reference and test data
+            (0=reference data, 1=test data).
 
         Returns
         -------
         Dictionary containing 'meta' and 'data' dictionaries.
         'meta' has the model's metadata.
         'data' contains the drift prediction and optionally the performance of the classifier
-            relative to its expectation under the no-change null.
+        relative to its expectation under the no-change null, the out-of-fold classifier model
+        prediction probabilities on the reference and test data.
         """
         # compute drift scores
-        p_val, dist = self.score(x)
+        p_val, dist, probs_ref, probs_test = self.score(x)
         drift_pred = int(p_val < self.p_val)
 
         # update reference dataset
@@ -229,10 +238,9 @@ class BaseClassifierDrift(BaseDetector):
             x = self.preprocess_fn(x)
         self.x_ref = update_reference(self.x_ref, x, self.n, self.update_x_ref)
         # used for reservoir sampling
-        self.n += x.shape[0]  # type: ignore
+        self.n += len(x)  # type: ignore
 
         # populate drift dict
-        # TODO: add instance level feedback
         cd = concept_drift_dict()
         cd['meta'] = self.meta
         cd['data']['is_drift'] = drift_pred
@@ -241,13 +249,16 @@ class BaseClassifierDrift(BaseDetector):
             cd['data']['threshold'] = self.p_val
         if return_distance:
             cd['data']['distance'] = dist
+        if return_probs:
+            cd['data']['probs_ref'] = probs_ref
+            cd['data']['probs_test'] = probs_test
         return cd
 
 
 class BaseMMDDrift(BaseDetector):
     def __init__(
             self,
-            x_ref: np.ndarray,
+            x_ref: Union[np.ndarray, list],
             p_val: float = .05,
             preprocess_x_ref: bool = True,
             update_x_ref: Optional[Dict[str, int]] = None,
@@ -308,16 +319,16 @@ class BaseMMDDrift(BaseDetector):
         self.preprocess_x_ref = preprocess_x_ref
         self.update_x_ref = update_x_ref
         self.preprocess_fn = preprocess_fn
-        self.n = x_ref.shape[0]  # type: ignore
+        self.n = len(x_ref)  # type: ignore
         self.n_permutations = n_permutations  # nb of iterations through permutation test
 
         # store input shape for save and load functionality
-        self.input_shape = input_shape if isinstance(input_shape, tuple) else x_ref.shape[1:]
+        self.input_shape = get_input_shape(input_shape, x_ref)
 
         # set metadata
         self.meta.update({'detector_type': 'offline', 'data_type': data_type})
 
-    def preprocess(self, x: np.ndarray) -> Tuple[np.ndarray, np.ndarray]:
+    def preprocess(self, x: Union[np.ndarray, list]) -> Tuple[np.ndarray, np.ndarray]:
         """
         Data preprocessing before computing the drift scores.
         Parameters
@@ -341,10 +352,10 @@ class BaseMMDDrift(BaseDetector):
         pass
 
     @abstractmethod
-    def score(self, x: np.ndarray) -> Tuple[float, float, np.ndarray]:
+    def score(self, x: Union[np.ndarray, list]) -> Tuple[float, float, np.ndarray]:
         pass
 
-    def predict(self, x: np.ndarray, return_p_val: bool = True, return_distance: bool = True) \
+    def predict(self, x: Union[np.ndarray, list], return_p_val: bool = True, return_distance: bool = True) \
             -> Dict[Dict[str, str], Dict[str, Union[int, float]]]:
         """
         Predict whether a batch of data has drifted from the reference data.
@@ -377,7 +388,7 @@ class BaseMMDDrift(BaseDetector):
             x = self.preprocess_fn(x)
         self.x_ref = update_reference(self.x_ref, x, self.n, self.update_x_ref)
         # used for reservoir sampling
-        self.n += x.shape[0]  # type: ignore
+        self.n += len(x)  # type: ignore
 
         # populate drift dict
         cd = concept_drift_dict()
@@ -395,7 +406,7 @@ class BaseMMDDrift(BaseDetector):
 class BaseLSDDDrift(BaseDetector):
     def __init__(
             self,
-            x_ref: np.ndarray,
+            x_ref: Union[np.ndarray, list],
             p_val: float = .05,
             preprocess_x_ref: bool = True,
             update_x_ref: Optional[Dict[str, int]] = None,
@@ -457,18 +468,18 @@ class BaseLSDDDrift(BaseDetector):
         self.preprocess_x_ref = preprocess_x_ref
         self.update_x_ref = update_x_ref
         self.preprocess_fn = preprocess_fn
-        self.n = x_ref.shape[0]  # type: ignore
+        self.n = len(x_ref)  # type: ignore
         self.n_permutations = n_permutations  # nb of iterations through permutation test
         self.n_kernel_centers = n_kernel_centers or max(self.n // 20, 1)
         self.lambda_rd_max = lambda_rd_max
 
         # store input shape for save and load functionality
-        self.input_shape = input_shape if isinstance(input_shape, tuple) else x_ref.shape[1:]
+        self.input_shape = get_input_shape(input_shape, x_ref)
 
         # set metadata
         self.meta.update({'detector_type': 'offline', 'data_type': data_type})
 
-    def preprocess(self, x: np.ndarray) -> Tuple[np.ndarray, np.ndarray]:
+    def preprocess(self, x: Union[np.ndarray, list]) -> Tuple[np.ndarray, np.ndarray]:
         """
         Data preprocessing before computing the drift scores.
         Parameters
@@ -487,10 +498,10 @@ class BaseLSDDDrift(BaseDetector):
             return self.x_ref, x
 
     @abstractmethod
-    def score(self, x: np.ndarray) -> Tuple[float, float, np.ndarray]:
+    def score(self, x: Union[np.ndarray, list]) -> Tuple[float, float, np.ndarray]:
         pass
 
-    def predict(self, x: np.ndarray, return_p_val: bool = True, return_distance: bool = True) \
+    def predict(self, x: Union[np.ndarray, list], return_p_val: bool = True, return_distance: bool = True) \
             -> Dict[Dict[str, str], Dict[str, Union[int, float]]]:
         """
         Predict whether a batch of data has drifted from the reference data.
@@ -519,11 +530,17 @@ class BaseLSDDDrift(BaseDetector):
         distance_threshold = np.sort(dist_permutations)[::-1][idx_threshold]
 
         # update reference dataset
-        if isinstance(self.update_x_ref, dict) and self.preprocess_fn is not None and self.preprocess_x_ref:
-            x = self.preprocess_fn(x)
+        if isinstance(self.update_x_ref, dict):
+            if self.preprocess_fn is not None and self.preprocess_x_ref:
+                x = self.preprocess_fn(x)
+                x = self._normalize(x)  # type: ignore
+            elif self.preprocess_fn is None:
+                x = self._normalize(x)  # type: ignore
+            else:
+                pass
         self.x_ref = update_reference(self.x_ref, x, self.n, self.update_x_ref)
         # used for reservoir sampling
-        self.n += x.shape[0]  # type: ignore
+        self.n += len(x)  # type: ignore
 
         # populate drift dict
         cd = concept_drift_dict()
@@ -541,7 +558,7 @@ class BaseLSDDDrift(BaseDetector):
 class BaseUnivariateDrift(BaseDetector):
     def __init__(
             self,
-            x_ref: np.ndarray,
+            x_ref: Union[np.ndarray, list],
             p_val: float = .05,
             preprocess_x_ref: bool = True,
             update_x_ref: Optional[Dict[str, int]] = None,
@@ -597,10 +614,10 @@ class BaseUnivariateDrift(BaseDetector):
         self.update_x_ref = update_x_ref
         self.preprocess_fn = preprocess_fn
         self.correction = correction
-        self.n = x_ref.shape[0]  # type: ignore
+        self.n = len(x_ref)  # type: ignore
 
         # store input shape for save and load functionality
-        self.input_shape = input_shape if isinstance(input_shape, tuple) else x_ref.shape[1:]
+        self.input_shape = get_input_shape(input_shape, x_ref)
 
         # compute number of features for the univariate tests
         if isinstance(n_features, int):
@@ -619,7 +636,7 @@ class BaseUnivariateDrift(BaseDetector):
         self.meta['detector_type'] = 'offline'  # offline refers to fitting the CDF for K-S
         self.meta['data_type'] = data_type
 
-    def preprocess(self, x: np.ndarray) -> Tuple[np.ndarray, np.ndarray]:
+    def preprocess(self, x: Union[np.ndarray, list]) -> Tuple[np.ndarray, np.ndarray]:
         """
         Data preprocessing before computing the drift scores.
 
@@ -643,7 +660,7 @@ class BaseUnivariateDrift(BaseDetector):
     def feature_score(self, x_ref: np.ndarray, x: np.ndarray) -> Tuple[np.ndarray, np.ndarray]:
         pass
 
-    def score(self, x: np.ndarray) -> Tuple[np.ndarray, np.ndarray]:
+    def score(self, x: Union[np.ndarray, list]) -> Tuple[np.ndarray, np.ndarray]:
         """
         Compute the feature-wise drift score which is the p-value of the
         statistical test and the test statistic.
@@ -661,7 +678,7 @@ class BaseUnivariateDrift(BaseDetector):
         score, dist = self.feature_score(x_ref, x)  # feature-wise univariate test
         return score, dist
 
-    def predict(self, x: np.ndarray, drift_type: str = 'batch',
+    def predict(self, x: Union[np.ndarray, list], drift_type: str = 'batch',
                 return_p_val: bool = True, return_distance: bool = True) \
             -> Dict[Dict[str, str], Dict[str, Union[np.ndarray, int, float]]]:
         """
@@ -706,7 +723,7 @@ class BaseUnivariateDrift(BaseDetector):
             x = self.preprocess_fn(x)
         self.x_ref = update_reference(self.x_ref, x, self.n, self.update_x_ref)
         # used for reservoir sampling
-        self.n += x.shape[0]  # type: ignore
+        self.n += len(x)  # type: ignore
 
         # populate drift dict
         cd = concept_drift_dict()
