@@ -1,7 +1,8 @@
 from .distance import mmd2, mmd2_from_kernel_matrix
-from .distance import relative_euclidean_distance, squared_pairwise_distance
+from .distance import relative_euclidean_distance, squared_pairwise_distance, permed_lsdds
 from .kernels import GaussianRBF
 from .prediction import predict_batch, predict_batch_transformer
+from .misc import zero_diag, quantile, subset_matrix
 
 __all__ = [
     "mmd2",
@@ -9,6 +10,10 @@ __all__ = [
     "relative_euclidean_distance",
     "squared_pairwise_distance",
     "GaussianRBF",
+    "permed_lsdds",
     "predict_batch",
-    "predict_batch_transformer"
+    "predict_batch_transformer",
+    "quantile",
+    "subset_matrix",
+    "zero_diag"
 ]
