@@ -149,6 +149,7 @@ def permed_lsdds(
     Vector of B LSDD estimates for each permutation, H_lam_inv which may have been inferred, and optionally
     the unpermed LSDD estimate.
     """
+
     # Compute (for each bootstrap) the average distance to each kernel center (Eqn 7)
     k_xc_perms = tf.stack([tf.gather(k_all_c, x_inds) for x_inds in x_perms], axis=0)
     k_yc_perms = tf.stack([tf.gather(k_all_c, y_inds) for y_inds in y_perms], axis=0)
@@ -161,12 +162,20 @@ def permed_lsdds(
         candidate_lambdas = [1/(4**i) for i in range(10)]  # TODO: More principled selection
         H_plus_lams = tf.stack([H+tf.eye(H.shape[0], dtype=H.dtype)*can_lam for can_lam in candidate_lambdas], axis=0)
         H_plus_lam_invs = tf.transpose(tf.linalg.inv(H_plus_lams), [1, 2, 0])  # lambdas last
-
         omegas = tf.einsum('jkl,bk->bjl', H_plus_lam_invs, h_perms)  # (Eqn 8)
         h_omegas = tf.einsum('bj,bjl->bl', h_perms, omegas)
         omega_H_omegas = tf.einsum('bkl,bkl->bl', tf.einsum('bjl,jk->bkl', omegas, H), omegas)
         rds = tf.reduce_mean(1 - (omega_H_omegas/h_omegas), axis=0)
-        lambda_index = int(tf.where(rds < lam_rd_max)[0])
+        less_than_rd_inds = tf.where(rds < lam_rd_max)
+        if len(less_than_rd_inds) == 0:
+            repeats = k_all_c.shape[0] - np.unique(k_all_c, dim=0).shape[0]
+            if repeats > 0:
+                msg = "Too many repeat instances for LSDD-based detection. \
+                Try using MMD-based detection instead"
+            else:
+                msg = "Unknown error. Try using MMD-based detection instead"
+            raise ValueError(msg)
+        lambda_index = int(less_than_rd_inds[0])
         lam = candidate_lambdas[lambda_index]
         logger.info(f"Using lambda value of {lam:.2g} with RD of {float(rds[lambda_index]):.2g}")
         H_plus_lam_inv = tf.linalg.inv(H+lam*tf.eye(H.shape[0], dtype=H.dtype))
